@@ -3,11 +3,12 @@ import asyncio
 import logging
 import pathlib
 import shutil
+from asyncio.exceptions import TimeoutError
 from typing import Callable, List, Optional
 
 import aiocron
-import aiohttp
-from pyunifiprotect import ProtectApiClient
+from aiohttp.client_exceptions import ClientPayloadError
+from pyunifiprotect import NvrError, ProtectApiClient
 from pyunifiprotect.data.nvr import Event
 from pyunifiprotect.data.types import EventType, ModelType
 from pyunifiprotect.data.websocket import WSAction, WSSubscriptionMessage
@@ -168,6 +169,7 @@ class UnifiProtectBackup:
         retention (str): How long should event clips be backed up for. Format as per the
                          `--max-age` argument of `rclone`
                          (https://rclone.org/filtering/#max-age-don-t-transfer-any-file-older-than-this)
+        rclone_args (str): Extra args passed directly to `rclone rcat`.
         ignore_cameras (List[str]): List of camera IDs for which to not backup events
         verbose (int): How verbose to setup logging, see :func:`setup_logging` for details.
         _download_queue (asyncio.Queue): Queue of events that need to be backed up
@@ -182,6 +184,7 @@ class UnifiProtectBackup:
         verify_ssl: bool,
         rclone_destination: str,
         retention: str,
+        rclone_args: str,
         ignore_cameras: List[str],
         verbose: int,
         port: int = 443,
@@ -200,6 +203,8 @@ class UnifiProtectBackup:
             retention (str): How long should event clips be backed up for. Format as per the
                             `--max-age` argument of `rclone`
                             (https://rclone.org/filtering/#max-age-don-t-transfer-any-file-older-than-this)
+            rclone_args (str): A bandwidth limit which is passed to the `--bwlimit` argument of
+                                   `rclone` (https://rclone.org/docs/#bwlimit-bandwidth-spec)
             ignore_cameras (List[str]): List of camera IDs for which to not backup events
             verbose (int): How verbose to setup logging, see :func:`setup_logging` for details.
         """
@@ -217,11 +222,13 @@ class UnifiProtectBackup:
         logger.debug(f"  {verify_ssl=}")
         logger.debug(f"  {rclone_destination=}")
         logger.debug(f"  {retention=}")
+        logger.debug(f"  {rclone_args=}")
         logger.debug(f"  {ignore_cameras=}")
         logger.debug(f"  {verbose=}")
 
         self.rclone_destination = rclone_destination
         self.retention = retention
+        self.rclone_args = rclone_args
 
         self._protect = ProtectApiClient(
             address,
@@ -250,6 +257,8 @@ class UnifiProtectBackup:
         # Start the pyunifiprotect connection by calling `update`
         logger.info("Connecting to Unifi Protect...")
         await self._protect.update()
+
+        # Get a mapping of camera ids -> names
         logger.info("Found cameras:")
         for camera in self._protect.bootstrap.cameras.values():
             logger.info(f" - {camera.id}: {camera.name}")
@@ -354,18 +363,17 @@ class UnifiProtectBackup:
 
         """
         while True:
-            event = await self._download_queue.get()
-            destination = self.generate_file_path(event)
-
-            logger.info(f"Backing up event: {event.id}")
-            logger.debug(f"Remaining Queue: {self._download_queue.qsize()}")
-            logger.debug(f"  Camera: {self._protect.bootstrap.cameras[event.camera_id].name}")
-            logger.debug(f"  Type: {event.type}")
-            logger.debug(f"  Start: {event.start.strftime('%Y-%m-%dT%H-%M-%S')}")
-            logger.debug(f"  End: {event.end.strftime('%Y-%m-%dT%H-%M-%S')}")
-            logger.debug(f"  Duration: {event.end-event.start}")
-
             try:
+                event = await self._download_queue.get()
+
+                logger.info(f"Backing up event: {event.id}")
+                logger.debug(f"Remaining Queue: {self._download_queue.qsize()}")
+                logger.debug(f"  Camera: {await self._get_camera_name(event.camera_id)}")
+                logger.debug(f"  Type: {event.type}")
+                logger.debug(f"  Start: {event.start.strftime('%Y-%m-%dT%H-%M-%S')}")
+                logger.debug(f"  End: {event.end.strftime('%Y-%m-%dT%H-%M-%S')}")
+                logger.debug(f"  Duration: {event.end-event.start}")
+
                 # Download video
                 logger.debug("  Downloading video...")
                 for x in range(5):
@@ -373,7 +381,7 @@ class UnifiProtectBackup:
                         video = await self._protect.get_camera_video(event.camera_id, event.start, event.end)
                         assert isinstance(video, bytes)
                         break
-                    except (AssertionError, aiohttp.client_exceptions.ClientPayloadError) as e:
+                    except (AssertionError, ClientPayloadError, TimeoutError) as e:
                         logger.warn(f"    Failed download attempt {x+1}, retying in 1s")
                         logger.exception(e)
                         await asyncio.sleep(1)
@@ -381,12 +389,14 @@ class UnifiProtectBackup:
                     logger.warn(f"Download failed after 5 attempts, abandoning event {event.id}:")
                     continue
 
+                destination = await self.generate_file_path(event)
+
                 logger.debug("  Uploading video via rclone...")
                 logger.debug(f"    To: {destination}")
                 logger.debug(f"    Size: {human_readable_size(len(video))}")
                 for x in range(5):
                     try:
-                        await self._upload_video(video, destination)
+                        await self._upload_video(video, destination, self.rclone_args)
                         break
                     except RcloneException as e:
                         logger.warn(f"    Failed upload attempt {x+1}, retying in 1s")
@@ -402,7 +412,7 @@ class UnifiProtectBackup:
                 logger.warn(f"Unexpected exception occurred, abandoning event {event.id}:")
                 logger.exception(e)
 
-    async def _upload_video(self, video: bytes, destination: pathlib.Path):
+    async def _upload_video(self, video: bytes, destination: pathlib.Path, rclone_args: str):
         """Upload video using rclone.
 
         In order to avoid writing to disk, the video file data is piped directly
@@ -411,12 +421,12 @@ class UnifiProtectBackup:
         Args:
             video (bytes): The data to be written to the file
             destination (pathlib.Path): Where rclone should write the file
+            rclone_args (str): Optional extra arguments to pass to `rclone`
 
         Raises:
             RuntimeError: If rclone returns a non-zero exit code
         """
-
-        cmd = f"rclone rcat -vv '{destination}'"
+        cmd = f"rclone rcat -vv {rclone_args} '{destination}'"
         proc = await asyncio.create_subprocess_shell(
             cmd,
             stdin=asyncio.subprocess.PIPE,
@@ -430,7 +440,7 @@ class UnifiProtectBackup:
         else:
             raise RcloneException(stdout.decode(), stderr.decode(), proc.returncode)
 
-    def generate_file_path(self, event: Event) -> pathlib.Path:
+    async def generate_file_path(self, event: Event) -> pathlib.Path:
         """Generates the rclone destination path for the provided event.
 
         Generates paths in the following structure:
@@ -449,7 +459,7 @@ class UnifiProtectBackup:
         """
         path = pathlib.Path(self.rclone_destination)
         assert isinstance(event.camera_id, str)
-        path /= self._protect.bootstrap.cameras[event.camera_id].name  # directory per camera
+        path /= await self._get_camera_name(event.camera_id)  # directory per camera
         path /= event.start.strftime("%Y-%m-%d")  # Directory per day
 
         file_name = f"{event.start.strftime('%Y-%m-%dT%H-%M-%S')} {event.type}"
@@ -462,3 +472,20 @@ class UnifiProtectBackup:
         path /= file_name
 
         return path
+
+    async def _get_camera_name(self, id: str):
+        try:
+            return self._protect.bootstrap.cameras[id].name
+        except KeyError:
+            # Refresh cameras
+            logger.debug(f"Unknown camera id: '{id}', checking API")
+
+            try:
+                await self._protect.update(force=True)
+            except NvrError:
+                logger.debug(f"Unknown camera id: '{id}'")
+                raise
+
+            name = self._protect.bootstrap.cameras[id].name
+            logger.debug(f"Found camera - {id}: {name}")
+            return name
